@@ -341,6 +341,9 @@ impl<O: Offset> Array for BinaryArray<O> {
     fn with_validity(&self, validity: Option<Bitmap>) -> Box<dyn Array> {
         Box::new(self.with_validity(validity))
     }
+    fn to_boxed(&self) -> Box<dyn Array> {
+        Box::new(self.clone())
+    }
 }
 
 unsafe impl<O: Offset> GenericBinaryArray<O> for BinaryArray<O> {
