@@ -31,7 +31,7 @@ impl<K: DictionaryKey, A: ffi::ArrowArrayRef> FromFfi<A> for DictionaryArray<K>
         let values = unsafe { array.buffer::<K>(1) }?;
 
         let data_type = K::PRIMITIVE.into();
-        let keys = PrimitiveArray::<K>::from_data(data_type, values, validity);
+        let keys = PrimitiveArray::<K>::try_new(data_type, values, validity)?;
         let values = array.dictionary()?.unwrap();
         let values = ffi::try_from(values)?.into();
 
