@@ -103,6 +103,9 @@ pub trait Array: Send + Sync {
     /// # Panic
     /// This function panics iff `validity.len() < self.len()`.
     fn with_validity(&self, validity: Option<Bitmap>) -> Box<dyn Array>;
+
+    /// Clone a `&dyn Array` to an owned `Box<dyn Array>`.
+    fn to_boxed(&self) -> Box<dyn Array>;
 }
 
 /// A trait describing a mutable array; i.e. an array whose values can be changed.
