@@ -21,14 +21,14 @@ impl<T: NativeType> PrimitiveArray<T> {
     /// # Implementation
     /// This does not assume that the iterator has a known length.
     pub fn from_values<I: IntoIterator<Item = T>>(iter: I) -> Self {
-        Self::from_data(T::PRIMITIVE.into(), Vec::<T>::from_iter(iter).into(), None)
+        Self::new(T::PRIMITIVE.into(), Vec::<T>::from_iter(iter).into(), None)
     }
 
     /// Creates a (non-null) [`PrimitiveArray`] from a slice of values.
     /// # Implementation
     /// This is essentially a memcopy
     pub fn from_slice<P: AsRef<[T]>>(slice: P) -> Self {
-        Self::from_data(
+        Self::new(
             T::PRIMITIVE.into(),
             Vec::<T>::from(slice.as_ref()).into(),
             None,
@@ -38,7 +38,7 @@ impl<T: NativeType> PrimitiveArray<T> {
     /// Creates a (non-null) [`PrimitiveArray`] from a vector of values.
     /// This does not have memcopy and is the fastest way to create a [`PrimitiveArray`].
     pub fn from_vec(array: Vec<T>) -> Self {
-        Self::from_data(T::PRIMITIVE.into(), array.into(), None)
+        Self::new(T::PRIMITIVE.into(), array.into(), None)
     }
 }
 
