@@ -355,10 +355,10 @@ where
         values.chain(null_indices.into_iter()).collect::<Vec<I>>()
     };
 
-    values.truncate(limit.unwrap_or_else(|| values.len()));
+    values.truncate(limit.unwrap_or(values.len()));
 
     let data_type = I::PRIMITIVE.into();
-    PrimitiveArray::<I>::from_data(data_type, values.into(), None)
+    PrimitiveArray::<I>::new(data_type, values.into(), None)
 }
 
 /// Compare two `Array`s based on the ordering defined in [ord](crate::array::ord).
