@@ -1,20 +1,18 @@
 use std::io::{Read, Seek, SeekFrom};
 use std::{collections::VecDeque, convert::TryInto};
 
-use arrow_format::ipc;
-use arrow_format::ipc::Message::{BodyCompression, CompressionType};
-
 use crate::buffer::Buffer;
 use crate::error::{ArrowError, Result};
-use crate::{bitmap::Bitmap, buffer::MutableBuffer, types::NativeType};
+use crate::{bitmap::Bitmap, types::NativeType};
 
 use super::super::compression;
 use super::super::endianess::is_native_little_endian;
+use super::{Compression, IpcBuffer, Node};
 
 fn read_swapped<T: NativeType, R: Read + Seek>(
     reader: &mut R,
     length: usize,
-    buffer: &mut MutableBuffer<T>,
+    buffer: &mut Vec<T>,
     is_little_endian: bool,
 ) -> Result<()> {
     // slow case where we must reverse bits
@@ -35,11 +33,12 @@ fn read_swapped<T: NativeType, R: Read + Seek>(
                 };
                 *slot = T::from_be_bytes(a);
                 Result::Ok(())
-            })
-            .unwrap();
+            })?;
     } else {
         // machine is big endian, file is little endian
-        todo!("reading little endian files from big endian machines not yet implemented.")
+        return Err(ArrowError::NotYetImplemented(
+            "Reading little endian files from big endian machines".to_string(),
+        ));
     }
     Ok(())
 }
@@ -49,7 +48,7 @@ fn read_uncompressed_buffer<T: NativeType, R: Read + Seek>(
     buffer_length: usize,
     length: usize,
     is_little_endian: bool,
-) -> Result<MutableBuffer<T>> {
+) -> Result<Vec<T>> {
     let bytes = length * std::mem::size_of::<T>();
     if bytes > buffer_length {
         return Err(ArrowError::OutOfSpec(
@@ -67,18 +66,12 @@ fn read_uncompressed_buffer<T: NativeType, R: Read + Seek>(
 
     // it is undefined behavior to call read_exact on un-initialized, https://doc.rust-lang.org/std/io/trait.Read.html#tymethod.read
     // see also https://github.com/MaikKlein/ash/issues/354#issue-781730580
-    let mut buffer = MutableBuffer::<T>::from_len_zeroed(length);
+    let mut buffer = vec![T::default(); length];
 
     if is_native_little_endian() == is_little_endian {
         // fast case where we can just copy the contents as is
-        unsafe {
-            // transmute T to bytes.
-            let slice = std::slice::from_raw_parts_mut(
-                buffer.as_mut_ptr() as *mut u8,
-                length * std::mem::size_of::<T>(),
-            );
-            reader.read_exact(slice)?;
-        }
+        let slice = bytemuck::cast_slice_mut(&mut buffer);
+        reader.read_exact(slice)?;
     } else {
         read_swapped(reader, length, &mut buffer, is_little_endian)?;
     }
@@ -90,8 +83,8 @@ fn read_compressed_buffer<T: NativeType, R: Read + Seek>(
     buffer_length: usize,
     length: usize,
     is_little_endian: bool,
-    compression: BodyCompression,
-) -> Result<MutableBuffer<T>> {
+    compression: Compression,
+) -> Result<Vec<T>> {
     if is_little_endian != is_native_little_endian() {
         return Err(ArrowError::NotYetImplemented(
             "Reading compressed and big endian IPC".to_string(),
@@ -100,46 +93,38 @@ fn read_compressed_buffer<T: NativeType, R: Read + Seek>(
 
     // it is undefined behavior to call read_exact on un-initialized, https://doc.rust-lang.org/std/io/trait.Read.html#tymethod.read
     // see also https://github.com/MaikKlein/ash/issues/354#issue-781730580
-    let mut buffer = MutableBuffer::<T>::from_len_zeroed(length);
+    let mut buffer = vec![T::default(); length];
 
     // decompress first
     // todo: move this allocation to an external buffer for re-use
     let mut slice = vec![0u8; buffer_length];
     reader.read_exact(&mut slice)?;
 
-    // Safety:
-    // This is safe because T is NativeType, which by definition can be transmuted to u8
-    let out_slice = unsafe {
-        std::slice::from_raw_parts_mut(
-            buffer.as_mut_ptr() as *mut u8,
-            length * std::mem::size_of::<T>(),
-        )
-    };
+    let out_slice = bytemuck::cast_slice_mut(&mut buffer);
 
-    match compression.codec() {
-        CompressionType::LZ4_FRAME => {
+    match compression.codec()? {
+        arrow_format::ipc::CompressionType::Lz4Frame => {
             compression::decompress_lz4(&slice[8..], out_slice)?;
             Ok(buffer)
         }
-        CompressionType::ZSTD => {
+        arrow_format::ipc::CompressionType::Zstd => {
             compression::decompress_zstd(&slice[8..], out_slice)?;
             Ok(buffer)
         }
-        _ => Err(ArrowError::NotYetImplemented(
-            "Compression format".to_string(),
-        )),
     }
 }
 
 pub fn read_buffer<T: NativeType, R: Read + Seek>(
-    buf: &mut VecDeque<&ipc::Schema::Buffer>,
+    buf: &mut VecDeque<IpcBuffer>,
     length: usize, // in slots
     reader: &mut R,
     block_offset: u64,
     is_little_endian: bool,
-    compression: Option<BodyCompression>,
+    compression: Option<Compression>,
 ) -> Result<Buffer<T>> {
-    let buf = buf.pop_front().unwrap();
+    let buf = buf
+        .pop_front()
+        .ok_or_else(|| ArrowError::oos("IPC: unable to fetch a buffer. The file is corrupted."))?;
 
     reader.seek(SeekFrom::Start(block_offset + buf.offset() as u64))?;
 
@@ -159,12 +144,19 @@ fn read_uncompressed_bitmap<R: Read + Seek>(
     length: usize,
     bytes: usize,
     reader: &mut R,
-) -> Result<MutableBuffer<u8>> {
-    // something is wrong if we can't `length`
-    assert!(length <= bytes * 8);
+) -> Result<Vec<u8>> {
+    if length > bytes * 8 {
+        return Err(ArrowError::OutOfSpec(format!(
+            "An array requires a bitmap with at least the same number of bits as slots. \
+            However, this array reports {} slots but the the bitmap in IPC only contains \
+            {} bits",
+            length,
+            bytes * 8,
+        )));
+    }
     // it is undefined behavior to call read_exact on un-initialized, https://doc.rust-lang.org/std/io/trait.Read.html#tymethod.read
     // see also https://github.com/MaikKlein/ash/issues/354#issue-781730580
-    let mut buffer = MutableBuffer::<u8>::from_len_zeroed(bytes);
+    let mut buffer = vec![0; bytes];
     reader.read_exact(buffer.as_mut_slice())?;
 
     Ok(buffer)
@@ -173,40 +165,39 @@ fn read_uncompressed_bitmap<R: Read + Seek>(
 fn read_compressed_bitmap<R: Read + Seek>(
     length: usize,
     bytes: usize,
-    compression: BodyCompression,
+    compression: Compression,
     reader: &mut R,
-) -> Result<MutableBuffer<u8>> {
-    let mut buffer = MutableBuffer::<u8>::from_len_zeroed((length + 7) / 8);
+) -> Result<Vec<u8>> {
+    let mut buffer = vec![0; (length + 7) / 8];
 
     // read all first
     // todo: move this allocation to an external buffer for re-use
     let mut slice = vec![0u8; bytes];
     reader.read_exact(&mut slice)?;
 
-    match compression.codec() {
-        CompressionType::LZ4_FRAME => {
+    match compression.codec()? {
+        arrow_format::ipc::CompressionType::Lz4Frame => {
             compression::decompress_lz4(&slice[8..], &mut buffer)?;
             Ok(buffer)
         }
-        CompressionType::ZSTD => {
+        arrow_format::ipc::CompressionType::Zstd => {
             compression::decompress_zstd(&slice[8..], &mut buffer)?;
             Ok(buffer)
         }
-        _ => Err(ArrowError::NotYetImplemented(
-            "Non LZ4 compressed IPC".to_string(),
-        )),
     }
 }
 
 pub fn read_bitmap<R: Read + Seek>(
-    buf: &mut VecDeque<&ipc::Schema::Buffer>,
+    buf: &mut VecDeque<IpcBuffer>,
     length: usize,
     reader: &mut R,
     block_offset: u64,
     _: bool,
-    compression: Option<BodyCompression>,
+    compression: Option<Compression>,
 ) -> Result<Bitmap> {
-    let buf = buf.pop_front().unwrap();
+    let buf = buf
+        .pop_front()
+        .ok_or_else(|| ArrowError::oos("IPC: unable to fetch a buffer. The file is corrupted."))?;
 
     reader.seek(SeekFrom::Start(block_offset + buf.offset() as u64))?;
 
@@ -222,12 +213,12 @@ pub fn read_bitmap<R: Read + Seek>(
 }
 
 pub fn read_validity<R: Read + Seek>(
-    buffers: &mut VecDeque<&ipc::Schema::Buffer>,
-    field_node: &ipc::Message::FieldNode,
+    buffers: &mut VecDeque<IpcBuffer>,
+    field_node: Node,
     reader: &mut R,
     block_offset: u64,
     is_little_endian: bool,
-    compression: Option<BodyCompression>,
+    compression: Option<Compression>,
 ) -> Result<Option<Bitmap>> {
     Ok(if field_node.null_count() > 0 {
         Some(read_bitmap(
@@ -239,7 +230,9 @@ pub fn read_validity<R: Read + Seek>(
             compression,
         )?)
     } else {
-        let _ = buffers.pop_front().unwrap();
+        let _ = buffers.pop_front().ok_or_else(|| {
+            ArrowError::oos("IPC: unable to fetch a buffer. The file is corrupted.")
+        })?;
         None
     })
 }
