@@ -78,6 +78,7 @@ fn infer_number(n: &serde_json::Number) -> DataType {
 }
 
 /// Coerce an heterogeneous set of [`DataType`] into a single one. Rules:
+/// * The empty set is coerced to `Null`
 /// * `Int64` and `Float64` are `Float64`
 /// * Lists and scalars are coerced to a list of a compatible scalar
 /// * Structs contain the union of all fields
@@ -85,6 +86,10 @@ fn infer_number(n: &serde_json::Number) -> DataType {
 pub(crate) fn coerce_data_type<A: Borrow<DataType>>(datatypes: &[A]) -> DataType {
     use DataType::*;
 
+    if datatypes.is_empty() {
+        return DataType::Null;
+    }
+
     let are_all_equal = datatypes.windows(2).all(|w| w[0].borrow() == w[1].borrow());
 
     if are_all_equal {
@@ -186,4 +191,15 @@ mod test {
             List(Box::new(Field::new(ITEM_NAME, Utf8, true))),
         );
     }
+
+    #[test]
+    fn test_coersion_of_nulls() {
+        assert_eq!(coerce_data_type(&[DataType::Null]), DataType::Null);
+        assert_eq!(
+            coerce_data_type(&[DataType::Null, DataType::Boolean]),
+            DataType::Utf8
+        );
+        let vec: Vec<DataType> = vec![];
+        assert_eq!(coerce_data_type(vec.as_slice()), DataType::Null);
+    }
 }
