@@ -152,6 +152,46 @@ where
                 |x: i64| x,
             )
         }
+        UInt8 => {
+            types.pop();
+            primitive::iter_to_arrays_nested(
+                columns.pop().unwrap(),
+                init.pop().unwrap(),
+                field.data_type().clone(),
+                chunk_size,
+                |x: i32| x as u8,
+            )
+        }
+        UInt16 => {
+            types.pop();
+            primitive::iter_to_arrays_nested(
+                columns.pop().unwrap(),
+                init.pop().unwrap(),
+                field.data_type().clone(),
+                chunk_size,
+                |x: i32| x as u16,
+            )
+        }
+        UInt32 => {
+            types.pop();
+            primitive::iter_to_arrays_nested(
+                columns.pop().unwrap(),
+                init.pop().unwrap(),
+                field.data_type().clone(),
+                chunk_size,
+                |x: i32| x as u32,
+            )
+        }
+        UInt64 => {
+            types.pop();
+            primitive::iter_to_arrays_nested(
+                columns.pop().unwrap(),
+                init.pop().unwrap(),
+                field.data_type().clone(),
+                chunk_size,
+                |x: i64| x as u64,
+            )
+        }
         Float32 => {
             types.pop();
             primitive::iter_to_arrays_nested(
@@ -181,6 +221,24 @@ where
                 chunk_size,
             )
         }
+        LargeUtf8 => {
+            types.pop();
+            binary::iter_to_arrays_nested::<i64, Utf8Array<i64>, _>(
+                columns.pop().unwrap(),
+                init.pop().unwrap(),
+                field.data_type().clone(),
+                chunk_size,
+            )
+        }
+        Binary => {
+            types.pop();
+            binary::iter_to_arrays_nested::<i32, BinaryArray<i32>, _>(
+                columns.pop().unwrap(),
+                init.pop().unwrap(),
+                field.data_type().clone(),
+                chunk_size,
+            )
+        }
         LargeBinary => {
             types.pop();
             binary::iter_to_arrays_nested::<i64, BinaryArray<i64>, _>(
@@ -190,7 +248,7 @@ where
                 chunk_size,
             )
         }
-        List(inner) => {
+        List(inner) | LargeList(inner) | FixedSizeList(inner, _) => {
             let iter = columns_to_iter_recursive(
                 vec![columns.pop().unwrap()],
                 types,
@@ -222,23 +280,6 @@ where
             let columns = columns.into_iter().rev().collect();
             Box::new(struct_::StructIterator::new(columns, fields.clone()))
         }
-        FixedSizeList(inner, _) => {
-            let iter = columns_to_iter_recursive(
-                vec![columns.pop().unwrap()],
-                types,
-                inner.as_ref().clone(),
-                init,
-                chunk_size,
-            )?;
-            let iter = iter.map(move |x| {
-                let (mut nested, array) = x?;
-                println!("{nested:?}");
-                println!("{array:?}");
-                let array = create_list(field.data_type().clone(), &mut nested, array)?;
-                Ok((nested, array))
-            });
-            Box::new(iter) as _
-        }
         other => todo!("{other:?}"),
     })
 }
