@@ -1,17 +1,18 @@
 use parquet2::{
     encoding::Encoding,
-    metadata::ColumnDescriptor,
+    metadata::Descriptor,
     page::DataPage,
-    statistics::{serialize_statistics, ParquetStatistics, PrimitiveStatistics, Statistics},
+    schema::types::PrimitiveType,
+    statistics::{serialize_statistics, PrimitiveStatistics},
     types::NativeType,
-    write::WriteOptions,
 };
 
 use super::super::utils;
+use super::super::WriteOptions;
 use crate::{
     array::{Array, PrimitiveArray},
     error::Result,
-    io::parquet::read::is_type_nullable,
+    io::parquet::read::schema::is_nullable,
     types::NativeType as ArrowNativeType,
 };
 
@@ -41,14 +42,14 @@ where
 pub fn array_to_page<T, R>(
     array: &PrimitiveArray<T>,
     options: WriteOptions,
-    descriptor: ColumnDescriptor,
+    descriptor: Descriptor,
 ) -> Result<DataPage>
 where
     T: ArrowNativeType,
     R: NativeType,
     T: num_traits::AsPrimitive<R>,
 {
-    let is_optional = is_type_nullable(descriptor.type_());
+    let is_optional = is_nullable(&descriptor.primitive_type.field_info);
 
     let validity = array.validity();
 
@@ -66,7 +67,10 @@ where
     encode_plain(array, is_optional, &mut buffer);
 
     let statistics = if options.write_statistics {
-        Some(build_statistics(array, descriptor.clone()))
+        Some(serialize_statistics(&build_statistics(
+            array,
+            descriptor.primitive_type.clone(),
+        )))
     } else {
         None
     };
@@ -74,6 +78,7 @@ where
     utils::build_plain_page(
         buffer,
         array.len(),
+        array.len(),
         array.null_count(),
         0,
         definition_levels_byte_length,
@@ -86,15 +91,15 @@ where
 
 pub fn build_statistics<T, R>(
     array: &PrimitiveArray<T>,
-    descriptor: ColumnDescriptor,
-) -> ParquetStatistics
+    primitive_type: PrimitiveType,
+) -> PrimitiveStatistics<R>
 where
     T: ArrowNativeType,
     R: NativeType,
     T: num_traits::AsPrimitive<R>,
 {
-    let statistics = &PrimitiveStatistics::<R> {
-        descriptor,
+    PrimitiveStatistics::<R> {
+        primitive_type,
         null_count: Some(array.null_count() as i64),
         distinct_count: None,
         max_value: array
@@ -113,6 +118,5 @@ where
                 x
             })
             .min_by(|x, y| x.ord(y)),
-    } as &dyn Statistics;
-    serialize_statistics(statistics)
+    }
 }
