@@ -1,22 +1,21 @@
-use parquet2::{
-    encoding::Encoding, metadata::ColumnDescriptor, page::DataPage, types::NativeType,
-    write::WriteOptions,
-};
+use parquet2::statistics::serialize_statistics;
+use parquet2::{encoding::Encoding, metadata::Descriptor, page::DataPage, types::NativeType};
 
 use super::super::levels;
 use super::super::utils;
+use super::super::WriteOptions;
 use super::basic::{build_statistics, encode_plain};
+use crate::io::parquet::read::schema::is_nullable;
 use crate::{
     array::{Array, Offset, PrimitiveArray},
     error::Result,
-    io::parquet::read::is_type_nullable,
     types::NativeType as ArrowNativeType,
 };
 
 pub fn array_to_page<T, R, O>(
     array: &PrimitiveArray<T>,
     options: WriteOptions,
-    descriptor: ColumnDescriptor,
+    descriptor: Descriptor,
     nested: levels::NestedInfo<O>,
 ) -> Result<DataPage>
 where
@@ -25,7 +24,7 @@ where
     T: num_traits::AsPrimitive<R>,
     O: Offset,
 {
-    let is_optional = is_type_nullable(descriptor.type_());
+    let is_optional = is_nullable(&descriptor.primitive_type.field_info);
 
     let validity = array.validity();
 
@@ -39,7 +38,10 @@ where
     encode_plain(array, is_optional, &mut buffer);
 
     let statistics = if options.write_statistics {
-        Some(build_statistics(array, descriptor.clone()))
+        Some(serialize_statistics(&build_statistics(
+            array,
+            descriptor.primitive_type.clone(),
+        )))
     } else {
         None
     };
@@ -47,6 +49,7 @@ where
     utils::build_plain_page(
         buffer,
         levels::num_values(nested.offsets()),
+        nested.offsets().len().saturating_sub(1),
         array.null_count(),
         repetition_levels_byte_length,
         definition_levels_byte_length,
