@@ -9,7 +9,7 @@ fn basics() {
 
 #[test]
 fn basics_u16_small() {
-    let mut iter = BitChunksExact::<u16>::new(&[0b11111111u8], 9);
+    let mut iter = BitChunksExact::<u16>::new(&[0b11111111u8], 7);
     assert_eq!(iter.next(), None);
     assert_eq!(iter.remainder(), 0b0000_0000_1111_1111u16);
 }
@@ -20,3 +20,14 @@ fn basics_u16() {
     assert_eq!(iter.next(), None);
     assert_eq!(iter.remainder(), 0b0000_0001_1111_1111u16);
 }
+
+#[test]
+fn remainder_u16() {
+    let mut iter = BitChunksExact::<u16>::new(
+        &[0b11111111u8, 0b00000001u8, 0b00000001u8, 0b11011011u8],
+        23,
+    );
+    assert_eq!(iter.next(), Some(511));
+    assert_eq!(iter.next(), None);
+    assert_eq!(iter.remainder(), 0b1101_1011_0000_0001u16);
+}
